#!/usr/bin/perl
# -----------------------------------------------------------------------------
#  ident "@(#)component.pl 1.2 03/09/30 SMI"
# -----------------------------------------------------------------------------
#
#  Copyright 2003 Sun Microsystems, Inc. All rights reserved.
#
# -----------------------------------------------------------------------------

package COMPONENT;

use strict;
use variables;

require "$ENV{NHINSTALL_LIB}/tools/lib/conf.pl";
require "$ENV{NHINSTALL_LIB}/tools/lib/common.pl";
require "$ENV{NHINSTALL_LIB}/tools/lib/sequencer.pl";

#==============================================================================
#
#    PACKAGES MANAGEMENT ROUTINES
#
#==============================================================================

#------------------------------------------------------------------------------
#
#  clean up when a package installation fails
#
#  1: admin file
#  2: answers file
#  3: node ip address
#
#------------------------------------------------------------------------------

sub exit_install_pkg {
  
  my ($admin_file, $answers_file, $node_ip, $package, $dir, $subdir) = @_;

  unlink($admin_file);
  if ($answers_file ne "") {
    unlink($answers_file) ;
  }
  COMMON::remote_exec ( $node_ip, "$RUMOUNT -f /mnt" );
  COMMON::error("Unable to install packages $package (from $dir/$subdir)" );
}

#------------------------------------------------------------------------------
#
#  remove a package from a MEN (errors are ignored)
#
#  1: Node ID
#  2: Package list
#  3: installed on local (LOCAL) or shared (SHARED) directory
#     or USR_SPECIFIC (/usr for diskless)
#
#------------------------------------------------------------------------------

sub remove_men_pkg {

  my ($node_id, $package_list, $option) = @_;

  my $node_ip = COMMON::get_external_ip($node_id);
  my $admin_file  = "$WORKING_DIR/admin";
  
  #
  #  configure settings depending installation is shared or not
  #
  my $basedir = "" ;
  my $optcmd  = "" ;
  my $adj     = "" ;
  my $loc     = "" ;

  if ($option eq "LOCAL") {

    $basedir = "default" ;
    $optcmd  = "" ;
    $adj     = "local" ;

  } elsif ($option eq "SHARED") {

    $basedir = "/$OPT_SUBDIR" ;
    $optcmd  = "-R $SHARED_FS" ;
    $adj     = "shared" ;

  } elsif ( $option eq "USR_SPECIFIC") {

    $loc = $SERVICE_USR_PATH;
    $basedir = $SERVICE_USR_BASEDIR;
    $optcmd  = "-R $loc" ;
    $adj     = "usr-specific" ;

  } else {
    COMMON::error("Invalid option ($option) for MEN package removing") ;
  }
  
  COMMON::print_action("Removing $adj packages: $package_list ...") ;

  # 
  # Create a pkgadd admin file
  #
  open(ADMIN, ">$admin_file") ||
    COMMON::error("Unable to create $admin_file");

   print ADMIN <<"FEO";
	
mail=
instance=overwrite
partial=nocheck
runlevel=quit
idepend=quit
rdepend=nocheck
space=quit
setuid=nocheck
conflict=nocheck
action=nocheck
basedir=$basedir

FEO


  close(ADMIN);
  
  #
  # copy the admin file on the remote node
  #
  COMMON::exec_cmd( "$RCP $admin_file $node_ip:/tmp/admin" ) ||
    COMMON::error( "Unable to copy $admin_file on $node_ip:/tmp/admin") ;
  
  #
  # remove the packages in the reverse order
  #
  # do not display error
  my @list = split(" ",$package_list) ;
  for (my $i = (scalar(@list) - 1) ; $i >= 0 ; $i--) {
    # remove the ending part of package name like SUNWefcx.u
    # (the package is known as SUNWefcx after installation)
    my $pkg = $list[$i];
    my $pos = index($pkg, ".");
    if ($pos > -1) {
      $pkg = substr($pkg, 0, $pos);
    }
    COMMON::remote_exec_ignore( $node_ip, "echo y | $RPKGRM $optcmd -a /tmp/admin $pkg" ) ;
  }

  #
  # remove the admin file from the remote node
  #
  COMMON::remote_exec ($node_ip, "$RM /tmp/admin" ) ;

  # remove the local adminfile
  unlink ($admin_file) ;
}

#------------------------------------------------------------------------------
#
# Install a package on a MEN 
#
# 1: Node ID
# 2: Package list
# 3: installed on 
#    . LOCAL:         /opt/SUNWcgha                 (basedir=default)
#    . SHARED:        /SUNWcgha/export/services     (basedir=/ha_v1/opt)
#    . USR_SPECIFIC:  /export/Solaris_x             (basedir=/usr_sparc.all
# 4: answers file (null string if not required)
#
# if exists, the answers file is deleted before exiting !
#
#------------------------------------------------------------------------------

sub install_men_pkg {

  my ($node_id, $root_dir, $package_subdir, $package_list, 
      $option, $answers_file) = @_;

  my $node_ip     = COMMON::get_external_ip($node_id);
  my $admin_file  = "$WORKING_DIR/admin";
  
  #
  #  configure settings
  #
  my $basedir = "default" ;
  my $dir     = "/opt/SUNWcgha" ;
  my $optcmd  = "" ;
  my $adj     = "" ;
  my $loc     = "" ;
  
  if ($option eq "LOCAL") {

    $basedir = "default" ;
    $dir     = "/opt/SUNWcgha" ;
    $optcmd  = "" ;
    $adj     = "local" ;

  } elsif ($option eq "SHARED") {

    $basedir = "/$OPT_SUBDIR" ;
    $dir     = "$SHARED_FS/$OPT_SUBDIR" ;
    $optcmd  = "-M -R $SHARED_FS" ;
    $adj     = "shared" ;

  } elsif ( $option eq "USR_SPECIFIC") {

    $loc = $SERVICE_USR_PATH;
    $basedir = $SERVICE_USR_BASEDIR;
    $dir     = "" ;
    $optcmd  = "-M -R $loc" ;
    $adj     = "usr-specific" ;

  } else {

    COMMON::error("Invalid option ($option) for MEN package installation") ;
  }

  COMMON::print_action("Installing $adj packages: $package_list ...") ;

  #
  # Test the product directory tree
  #
  COMMON::control_dir_is_readable ("$root_dir/$package_subdir");
  
  #
  # mount ROOT_DIR
  #   
  COMMON::remote_mount($node_ip, $root_dir, "/mnt") ;
  
  # 
  # Create a pkgadd admin file
  #
  open(ADMIN, ">$admin_file") ||
    COMMON::error("Unable to create $admin_file");

   print ADMIN <<"FEO";
	
mail=
instance=overwrite
partial=nocheck
runlevel=quit
idepend=nocheck
rdepend=nocheck
space=quit
setuid=nocheck
conflict=nocheck
action=nocheck
basedir=$basedir

FEO


  close(ADMIN);
  
  #
  # copy the admin file on the remote node
  #
  COMMON::exec_cmd( "$RCP $admin_file $node_ip:/tmp/admin" ) ||
    COMMON::error ( "Unable to copy $admin_file on $node_ip:/tmp/admin") ;
  
  #
  # copy the answers file on the remote node
  #
  if ($answers_file ne "") {
    COMMON::exec_cmd( "$RCP $answers_file $node_ip:/tmp/answers" ) ||
      COMMON::error( "Unable to copy $answers_file on $node_ip:/tmp/answers") ;
  }

  #
  # Remotely create the install directory in case it is needed by BASE_DIR
  #
  if ($dir ne "") {
    COMMON::remote_exec ($node_ip, "$RMKDIR -p $dir" ) ||
      COMMON::error("Unable to create $dir directory needed by BASE_DIR package installation setup");
  }

  #
  # Install packages remotely
  #
  if ($answers_file eq "") {
    COMMON::remote_exec ( "$node_ip", "$RPKGADD -n -a /tmp/admin -d /mnt/$package_subdir $optcmd $package_list" ) ||
      exit_install_pkg($admin_file, $answers_file, $node_ip, $package_list, $root_dir, $package_subdir);
  } else {
    COMMON::remote_exec ( "$node_ip", "$RPKGADD -n -a /tmp/admin -r /tmp/answers -d /mnt/$package_subdir $optcmd $package_list" ) ||
      exit_install_pkg($admin_file, $answers_file, $node_ip, $package_list, $root_dir, $package_subdir);
  }
  
  #
  # remove the admin file from the remote node
  #
  COMMON::remote_exec ( "$node_ip", "$RM /tmp/admin" ) ||
    exit_install_pkg($admin_file, $answers_file, $node_ip, $package_list, $root_dir, $package_subdir);
  
  #
  # remove the answers file from the remote node
  #
  if ($answers_file ne "") {
    COMMON::remote_exec ( "$node_ip", "$RM /tmp/answers" ) ||
      exit_install_pkg($admin_file, $answers_file, $node_ip, $package_list, $root_dir, $package_subdir);
  }

  #
  # remove previously created pkgadd admin and answers file
  #
  unlink($admin_file);
  if ($answers_file ne "") {
    unlink($answers_file);
  }

  #
  # umount ROOT_DIR
  #   
  COMMON::remote_exec ( "$node_ip", "$RUMOUNT -f /mnt" ) ||
    COMMON::error ( "Unable to NFS umount /mnt" );
}

#------------------------------------------------------------------------------
#
# Remove packages for all NMENs on a MEN
#
# 1: Node ID
# 2: Package list
#
#------------------------------------------------------------------------------

sub remove_nmen_pkg {

  my ($nodeid, $package_list) = @_;

  my $node_ip     = COMMON::get_external_ip($nodeid);
  my $admin_file  = "$WORKING_DIR/admin";
  
  COMMON::print_action("Remove packages for diskless nodes: $package_list ...");

  # 
  # Create a pkgrm admin file
  #
  open(ADMIN, ">$admin_file") ||
    COMMON::error("Unable to create $admin_file");
  
  my $basedir="default" ;

   print ADMIN <<"FEO";
	
mail=
instance=overwrite
partial=nocheck
runlevel=quit
idepend=quit
rdepend=nocheck
space=quit
setuid=nocheck
conflict=nocheck
action=nocheck
basedir=$basedir

FEO


  close(ADMIN);
  
  #
  # copy the admin file on the remote node
  #
  COMMON::exec_cmd( "$RCP $admin_file $node_ip:/tmp/admin" ) ||
    COMMON::error ( "Unable to copy $admin_file on $node_ip:/tmp/admin") ;

  #
  # remove packages for all NMEN concerned by the installation
  #
  for my $node (0..$#NODE_LIST) {

    if (($NODE_LIST[$node]{type} eq "DISKLESS") && ($NODE_LIST[$node]{concerned})) {

      my $node_name_template = COMMON::build_node_name($NODE_LIST[$node]{id}, "");
      my $node_root = "/export/root/$node_name_template";

      COMMON::print_subaction("for $node_name_template ...") ;
      #
      # remove the packages in the reverse order
      #
      # do not display error
      my @list = split(" ",$package_list) ;
      for (my $i = (scalar(@list) - 1) ; $i >= 0 ; $i--) {
	# remove the ending part of package name like SUNWefcx.u
	# (the package is known as SUNWefcx after installation)
	my $pkg = $list[$i];
	my $pos = index($pkg, ".");
	if ($pos > -1) {
	  $pkg = substr($pkg, 0, $pos);
	}
	COMMON::remote_exec_ignore ( "$node_ip", "echo y | $RPKGRM -a /tmp/admin -R $node_root $pkg" ) ;
      }
    }
  }
  
  #
  # remove the admin file from the remote node
  #
  COMMON::remote_exec ( "$node_ip", "$RM /tmp/admin" ) ;
  
  # remove previously created pkgadd admin file
  #
  unlink($admin_file);	
}

#------------------------------------------------------------------------------
#
# Install a package for all NMENs on a MEN
#
# 1: Node ID
# 2: Package list
# 3: answers file (null string if not required)
#
# if exists, the answers file is deleted before exiting !
#
#------------------------------------------------------------------------------

sub install_nmen_pkg {

  my ($node_id, $root_dir, $package_subdir, $package_list, $answers_file) = @_;

  my $node_ip     = COMMON::get_external_ip($node_id);
  my $admin_file  = "$WORKING_DIR/admin";
  
  #
  # Test the product directory tree
  #
  COMMON::control_dir_is_readable ("$root_dir/$package_subdir");
  
  COMMON::print_action("Installing packages for diskless nodes: $package_list ...") ;

  #
  # mount ROOT_DIR
  #   
  COMMON::remote_mount($node_ip, $root_dir, "/mnt") ;
  
  # 
  # Create a pkgadd admin file
  #
  open(ADMIN, ">$admin_file") ||
    COMMON::error("Unable to create $admin_file");
  
  my $basedir="default" ;

   print ADMIN <<"FEO";
	
mail=
instance=overwrite
partial=nocheck
runlevel=quit
idepend=nocheck
rdepend=nocheck
space=quit
setuid=nocheck
conflict=nocheck
action=nocheck
basedir=$basedir

FEO


  close(ADMIN);
  
  #
  # copy the admin file on the remote node
  #
  COMMON::exec_cmd( "$RCP $admin_file $node_ip:/tmp/admin" ) ||
    COMMON::error ( "Unable to copy $admin_file on $node_ip:/tmp/admin") ;
  
  #
  # copy the answers file on the remote node
  #
  if ($answers_file ne "") {
    COMMON::exec_cmd( "$RCP $answers_file $node_ip:/tmp/answers" ) ||
	  COMMON::error ( "Unable to copy $answers_file on $node_ip:/tmp/answers") ;
  }

  #
  # Install packages on NMEN concerned by the installation
  #
  my $node;
  foreach $node (0..$#NODE_LIST) {
    if (($NODE_LIST[$node]{type} eq "DISKLESS") && ($NODE_LIST[$node]{concerned})) {
      my $node_name_template = COMMON::build_node_name($NODE_LIST[$node]{id}, "");
      my $node_root = "/export/root/$node_name_template";

      #
      # Remotely create the install directory in case it is needed by BASE_DIR
      #
      COMMON::remote_exec ( "$node_ip", "$RMKDIR -p $node_root/opt/SUNWcgha " ) ||
	COMMON::error("Unable to create $node_root/opt/SUNWcgha directory needed by BASE_DIR package installation setup");	
      COMMON::print_subaction("for $node_name_template ...") ;
      #
      # Install packages remotely
      #
      if ($answers_file eq "") {
	COMMON::remote_exec ( "$node_ip", "$RPKGADD -n -a /tmp/admin -d /mnt/$package_subdir -M -R $node_root $package_list" ) ||
	  exit_install_pkg($admin_file, $answers_file, $node_ip, $package_list, $root_dir, $package_subdir);
      } else {
	COMMON::remote_exec ( "$node_ip", "$RPKGADD -n -a /tmp/admin -r /tmp/answers -d /mnt/$package_subdir -M -R $node_root $package_list" ) ||
	  exit_install_pkg($admin_file, $answers_file, $node_ip, $package_list, $root_dir, $package_subdir);
      }
    }
  } 

  
  #
  # remove the admin file from the remote node
  #
  COMMON::remote_exec ( "$node_ip", "$RM /tmp/admin" ) ||
    exit_install_pkg($admin_file, $answers_file, $node_ip, $package_list, $root_dir, $package_subdir);
  
  #
  # remove the answers file from the remote node
  #
  if ($answers_file ne "") {
    COMMON::remote_exec ( "$node_ip", "$RM /tmp/answers" ) ||
      exit_install_pkg($admin_file, $answers_file, $node_ip, $package_list, $root_dir, $package_subdir);
  }

  #
  # remove previously created pkgadd admin and answers file
  #
  unlink($admin_file);
  if ($answers_file ne "") {
    unlink($answers_file);
  }

  #
  # umount ROOT_DIR
  #   
  COMMON::remote_exec ( "$node_ip", "$RUMOUNT -f /mnt" ) ||
    COMMON::error ( "Unable to NFS umount /mnt" );	
}

#==============================================================================
#
#    PATCH MANAGEMENT ROUTINES
#
#==============================================================================

#------------------------------------------------------------------------------
#
# Create the prerequisites for patching shared packages
#
#------------------------------------------------------------------------------

sub shared_patch_preparation {
  my $dir  = "var/sadm/system/admin" ;
  my $file = "INST_RELEASE" ;

  my $node_ip   = COMMON::get_external_ip($NODE_ID);

  COMMON::print_action("Creating prerequisites for patching shared packages ...") ;

  #
  # Remotely create directory
  #
  COMMON::remote_exec( $node_ip, "$RMKDIR -p $DIR_SUNWCGHA_EXPORTED/services/$dir" ) ||
    COMMON::error( "Unable to create $DIR_SUNWCGHA_EXPORTED/services/$dir" );

  #
  # Remotely copy file
  #
  COMMON::remote_exec($node_ip, "$RCOPY /$dir/$file $DIR_SUNWCGHA_EXPORTED/services/$dir/$file") ||
    COMMON::error( "Unable to copy $file on $DIR_SUNWCGHA_EXPORTED/services/$dir/$file" );
}

#------------------------------------------------------------------------------
#
#  clean up when a patch installation fails
#
#  1: node ip address
#
#------------------------------------------------------------------------------

sub exit_install_patch {

  my ($node_ip, $patch_id, $dir, $subdir) = @_;

  COMMON::remote_exec ( "$node_ip", "$RUMOUNT -f /mnt" );
  COMMON::error("Unable to install patch $patch_id (from $dir/$subdir)" );
}

#------------------------------------------------------------------------------
#
# remove patch on a MEN
# 
#------------------------------------------------------------------------------

sub remove_men_patch {

  my ($node_id, $patch, $option) = @_;

  my $node_ip     = COMMON::get_external_ip($node_id);
  my $optcmd ;
  my $adj ;

  my $patchid = $patch;
  if (substr($patch,0,1) eq "T") {
    # it's a temporary patch, remove the T to have the patch ID
    $patchid = substr($patch,1,length($patch)-1);
  }

  if ($option eq "LOCAL") {
    $optcmd = "" ;
    $adj = "local" ;
  } elsif ($option eq "SHARED") {
    $optcmd = "-R $SHARED_FS" ;
    $adj = "shared" ;
  } else {
    COMMON::error("Invalid option ($option) for MEN patch installation") ;
  }
  COMMON::print_action("Removing $adj patch: $patch ...") ;	
		
  #
  # Remove patch remotely
  #
  my @list = split(" ",$patch) ;
  for (my $i = (scalar(@list) - 1) ; $i >= 0 ; $i--) {
    COMMON::remote_exec_ignore ( "$node_ip", "$RPATCHRM $optcmd $list[$i]" ) ;
  }
}

#------------------------------------------------------------------------------
#
# install patch on a MEN
# 
#------------------------------------------------------------------------------

sub install_men_patch {

  my ($node_id, $root_dir, $patch_subdir, $patch, $option) = @_;

  my $node_ip     = COMMON::get_external_ip($node_id);
  my $optcmd ;
  my $adj ;

  if ($option eq "LOCAL") {
    $optcmd = "" ;
    $adj = "local" ;
  } elsif ($option eq "SHARED") {
    $optcmd = "-R $SHARED_FS" ;
    $adj = "shared" ;
  } else {
    COMMON::error("Invalid option ($option) for MEN patch installation") ;
  }

  COMMON::print_action("Installing $adj patch: $patch ...") ;

  #
  # Test the product directory tree
  #
  COMMON::control_dir_is_readable ("$root_dir/$patch_subdir");

  #
  # mount ROOT_DIR
  #   
  COMMON::remote_mount($node_ip, $root_dir, "/mnt") ;
		
  #
  # Add patch remotely
  #
  COMMON::remote_exec ( "$node_ip", "$RPATCHADD -M /mnt/$patch_subdir $optcmd $patch" ) ||
    exit_install_patch($node_ip, $patch, $root_dir, $patch_subdir);
      
  #
  # umount ROOT_DIR
  #   
  COMMON::remote_exec ( "$node_ip", "$RUMOUNT -f /mnt" ) || 
    COMMON::error ( "Unable to NFS umount /mnt" );
}

#------------------------------------------------------------------------------
#
# remove a patch for all NMENs on a MEN
# 
#------------------------------------------------------------------------------

sub remove_nmen_patch {

  my ($node_id, $patch, $where) = @_;

  my $node_ip     = COMMON::get_external_ip($node_id);
   
  COMMON::print_action("Removing patch for diskless nodes: $patch ...") ;
		
  if (($where ne "SOLARIS") && ($where ne "DISKLESS")) {
    COMMON::error("internal error: install_nmen_patch location: $where (SOLARIS or DISKLESS required)");
  }
  
  my $patchid = $patch;
  if (substr($patch,0,1) eq "T") {
    # it's a temporary patch, remove the T to have the patch ID
    $patchid = substr($patch,1,length($patch)-1);
  }

  if ($where eq "SOLARIS") {
    #
    # Remove patch from Solaris environment remotely
    #
    COMMON::print_subaction("for $TARGET_OS service ...") ;
    my @list = split(" ",$patch) ;
    for (my $i = (scalar(@list) - 1) ; $i >= 0 ; $i--) {
      COMMON::remote_exec_ignore ( "$node_ip", "$RPATCHRM -S $TARGET_OS $list[$i]" ) ;
    }
  } else {

    #
    # Remove patches from diskless nodes remotely
    #
    for my $node (0..$#NODE_LIST) {
      if (($NODE_LIST[$node]{type} eq "DISKLESS") && ($NODE_LIST[$node]{concerned})) {
	my $node_name_template = COMMON::build_node_name($NODE_LIST[$node]{id}, "");
	my $node_root = "/export/root/$node_name_template";
	
	COMMON::print_subaction("for $node_name_template ...") ;
	
	my @list = split(" ",$patch) ;
	for (my $i = (scalar(@list) - 1) ; $i >= 0 ; $i--) {
	 COMMON::remote_exec_ignore ( "$node_ip", "$RPATCHRM -R $node_root $list[$i]" ) ;
	}
      }
    }
  }
}

#------------------------------------------------------------------------------
#
# install patch for all NMENs on a MEN
# 
#------------------------------------------------------------------------------

sub install_nmen_patch {

  my ($node_id, $root_dir, $patch_subdir, $patch, $where) = @_;

  my $node_ip     = COMMON::get_external_ip($node_id);
   
  COMMON::print_action("Installing patch for diskless nodes: $patch ...") ;

  #
  # Test the product directory tree
  #
  COMMON::control_dir_is_readable ("$root_dir/$patch_subdir");

  #
  # mount ROOT_DIR
  #
  COMMON::remote_mount($node_ip, $root_dir, "/mnt") ;

  if (($where ne "SOLARIS") && ($where ne "DISKLESS")) {
    COMMON::error("internal error: install_nmen_patch location: $where (SOLARIS or DISKLESS required)");
  }

  if ($where eq "SOLARIS") {

    #
    # Add patch on Solaris environment remotely
    #
    COMMON::print_subaction("for $TARGET_OS service ...") ;
    if ($USE_DCSS) {
      COMMON::remote_exec ( "$node_ip", "$RNETRAOS patch -i -f -M /mnt/$patch_subdir -P $patch $TARGET_OS" ) ||
	exit_install_patch($node_ip, $patch, $root_dir, $patch_subdir);
    } else {
      COMMON::remote_exec ( "$node_ip", "$RPATCHADD -M /mnt/$patch_subdir -S $TARGET_OS $patch" ) ||
	exit_install_patch($node_ip, $patch, $root_dir, $patch_subdir);
    }

  } else {

    #
    # Install patches on diskless nodes remotely
    #
    for my $node (0..$#NODE_LIST) {
      if (($NODE_LIST[$node]{type} eq "DISKLESS") && ($NODE_LIST[$node]{concerned})) {
	my $node_name_template = COMMON::build_node_name($NODE_LIST[$node]{id}, "");
	my $node_root = "/export/root/$node_name_template";
	
	COMMON::print_subaction("for $node_name_template ...") ;
	
	COMMON::remote_exec ( "$node_ip", "$RPATCHADD -R $node_root -M /mnt/$patch_subdir $patch" ) ||
	  exit_install_patch($node_ip, $patch, $root_dir, $patch_subdir);
      }
    }
  }
   
  #
  # umount ROOT_DIR
  #   
  COMMON::remote_exec ( "$node_ip", "$RUMOUNT -f /mnt" ) || 
    COMMON::error ( "Unable to NFS umount /mnt" );
}

#==============================================================================
#
#    PATCH MANAGEMENT ROUTINES USING SMOSSERVICE
#
#==============================================================================

#------------------------------------------------------------------------------
#
# add a list of patches to the spool directory
#
#------------------------------------------------------------------------------

sub install_sm_nmen_patch {

  my ($node_id, $root_dir, $patch_subdir, $patch, $where) = @_;

  my $node_ip     = COMMON::get_external_ip($node_id);
   
  COMMON::print_action("Adding patch to the spool directory: $patch ...") ;

  #
  # Test the product directory tree
  #
  COMMON::control_dir_is_readable ("$root_dir/$patch_subdir");

  #
  # mount ROOT_DIR
  #
  COMMON::remote_mount($node_ip, $root_dir, "/mnt") ;

  #
  # add the patches to the spool area
  #
  my @list = split(" ",$patch) ;
  for (my $i = 0; $i < (scalar(@list)) ; $i++) {
    COMMON::remote_exec("$node_ip", 
			"$RSMOSSERVICE patch -p $PASSWORD -- " .
			"-a /mnt/$patch_subdir/$list[$i]" ) 
      || exit_install_patch($node_ip, $list[$i], $root_dir, $patch_subdir);
    SEQUENCER::set_data("AT_LEAST_ONE_SM_PATCH", $TRUE);
  }

  #
  # umount ROOT_DIR
  #   
  COMMON::remote_exec ( "$node_ip", "$RUMOUNT -f /mnt" ) || 
    COMMON::error ( "Unable to NFS umount /mnt" );
}

#------------------------------------------------------------------------------
#
# remove a list of patch from the spool directory
#
#------------------------------------------------------------------------------

sub remove_sm_nmen_patch {

  my ($node_id, $patch, $where) = @_;

  my $node_ip     = COMMON::get_external_ip($node_id);
   
  COMMON::print_action("Removing patch from the spool directory: $patch ...") ;


  #
  # remove the patches from the spool area
  #
  my @list = split(" ",$patch) ;
  for (my $i = (scalar(@list) - 1) ; $i >= 0 ; $i--) {

    COMMON::remote_exec_ignore("$node_ip", 
			       "$RSMOSSERVICE patch -p $PASSWORD -- " .
			       "-r $list[$i]" ) 
  }
  SEQUENCER::set_data("AT_LEAST_ONE_SM_PATCH", $FALSE);
}

#------------------------------------------------------------------------------
#
# synchronize the patch 
#
#------------------------------------------------------------------------------

sub sync_sm_nmen_patch {

  my ($node_id) = @_;

  my $node_ip     = COMMON::get_external_ip($node_id);

  # can be TRUE, FALSE or missing (meaning FALSE)
  if (SEQUENCER::get_data("AT_LEAST_ONE_SM_PATCH") ne $TRUE) {
    return;
  }
   
  COMMON::print_action("Synchronizing spooled patch and updating diskless ...");
  #
  # updating all the diskless
  #
  COMMON::remote_exec("$node_ip", 
		      "$RSMOSSERVICE patch -p $PASSWORD -- -m -U" ) 
    || COMMON::error("Unable to update diskless with patches" );

  SEQUENCER::set_data("AT_LEAST_ONE_SM_PATCH", $FALSE);
}

#==============================================================================
#
# INSTALLATION OF PACKAGES AND PATCHES BASED ON A COMPONENT DESCRIPTION
#
#==============================================================================

#------------------------------------------------------------------------------
#
# get the component reference
#
#------------------------------------------------------------------------------

sub get_component_idx {

  my ($component) = @_;

  for my $i (0..$#COMPONENT_LIST) {
    if ($COMPONENT_LIST[$i]{name} eq $component) {
      return $i;
    }
  }

  my $release = SEQUENCER::get_data("RELEASE");
  COMMON::error("COMPONENT $component unknown for OS release $release");
}

#------------------------------------------------------------------------------
#
# get the location reference
#
#------------------------------------------------------------------------------

sub get_location_idx {

  my ($location) = @_;

  for my $i (0..$#LOCATION_LIST) {
    if ($LOCATION_LIST[$i]{name} eq $location) {
      return $i;
    }
  }

  my $release = SEQUENCER::get_data("RELEASE");
  COMMON::error("LOCATION $location unknown for OS release $release");
}

#------------------------------------------------------------------------------
#
# get the distrib root
#
#------------------------------------------------------------------------------

sub get_distrib_root {

  my ($location) = @_;

  if ($location eq "NHAS") {
    return $NHAS_DIR;
  } elsif ($location eq "SOLARIS") {
    return $SOLARIS_DIR;
  }
}

#------------------------------------------------------------------------------
#
# return component information for master-eligible node
#
#------------------------------------------------------------------------------

sub get_info_men_component {

  my ($component, $info) = @_;

  my $idx = get_component_idx($component);

  if ($info eq "DIR") {
    return $COMPONENT_LIST[$idx]{dir};
  } elsif ($info eq "PACKAGE") {
    return $COMPONENT_LIST[$idx]{pkg_men};
  } else {
    COMMON::error("INTERNAL: invalid info $info for component $component");
  }
    
}


#------------------------------------------------------------------------------
#
# installation of a component on a master-eligible node
#
#------------------------------------------------------------------------------

sub install_men_component {

  my ($node_id, $component, $answer_file) = @_;

  my $idx = get_component_idx($component);
  my $loc = get_location_idx($COMPONENT_LIST[$idx]{location});

  my $scope = $COMPONENT_LIST[$idx]{scope};
  if ($scope eq "NORMAL") {
    $scope = "LOCAL";
  }

  my $root = get_distrib_root($COMPONENT_LIST[$idx]{location});

  if ($COMPONENT_LIST[$idx]{pkg_men} ne "") {
    install_men_pkg($node_id, 
		    $root, 
		    $LOCATION_LIST[$loc]{package}, 
		    $COMPONENT_LIST[$idx]{pkg_men}, 
		    $scope, 
		    $answer_file);
  }

  if ($COMPONENT_LIST[$idx]{patch_men} ne "") {
    install_men_patch($node_id, 
		      $root,
		      $LOCATION_LIST[$loc]{patch} , 
		      $COMPONENT_LIST[$idx]{patch_men},
		      $scope) ;
  }
}

#------------------------------------------------------------------------------
#
# remove a component from a master-eligible node
#
#------------------------------------------------------------------------------

sub remove_men_component {

  my ($node_id, $component) = @_;

  my $idx = get_component_idx($component);

  my $scope = $COMPONENT_LIST[$idx]{scope};
  if ($scope eq "NORMAL") {
    $scope = "LOCAL";
  }

  if ($COMPONENT_LIST[$idx]{patch_men} ne "") {
    remove_men_patch($node_id, $COMPONENT_LIST[$idx]{patch_men}, $scope) ;
  }

  if ($COMPONENT_LIST[$idx]{pkg_men} ne "") {
    remove_men_pkg($node_id, $COMPONENT_LIST[$idx]{pkg_men}, $scope);
  }

}

#------------------------------------------------------------------------------
#
# installation of a diskless component
#
#------------------------------------------------------------------------------

sub install_nmen_component {

  my ($node_id, $component, $answer_file) = @_;

  my $idx = get_component_idx($component);
  my $loc = get_location_idx($COMPONENT_LIST[$idx]{location});

  my $scope = $COMPONENT_LIST[$idx]{scope};
  if ($scope eq "SHARED") {
    return;
  }

  my $root = get_distrib_root($COMPONENT_LIST[$idx]{location});

  #
  # install on the Solaris services (usr_specific)
  # during a full cluster install
  #
  if ($INSTALL_TYPE eq "CLUSTER") {

    if ($COMPONENT_LIST[$idx]{pkg_usr} ne "") {
      install_men_pkg($node_id, 
		      $root,
		      $LOCATION_LIST[$loc]{package},
		      $COMPONENT_LIST[$idx]{pkg_usr}, 
		      "USR_SPECIFIC", 
		      $answer_file);
    }
    
    if ($COMPONENT_LIST[$idx]{patch_usr} ne "") {
      install_nmen_patch($node_id, 
			$root,
			$LOCATION_LIST[$loc]{patch} , 
			$COMPONENT_LIST[$idx]{patch_usr},
			"SOLARIS") ;
    }
  }

  #
  # install on the root of each diskless
  # 
  if ($COMPONENT_LIST[$idx]{pkg_root} ne "") {
    install_nmen_pkg($node_id, 
		    $root,
		    $LOCATION_LIST[$loc]{package},
		    $COMPONENT_LIST[$idx]{pkg_root},
		    $answer_file);
  }
    
  if ($COMPONENT_LIST[$idx]{patch_root} ne "") {
    install_nmen_patch($node_id, 
		       $root,
		       $LOCATION_LIST[$loc]{patch} , 
		       $COMPONENT_LIST[$idx]{patch_root},
		       "DISKLESS") ;
  }
}

#------------------------------------------------------------------------------
#
# remove a diskless component
#
#------------------------------------------------------------------------------

sub remove_nmen_component {

  my ($node_id, $component) = @_;

  my $idx = get_component_idx($component);
  my $loc = get_location_idx($COMPONENT_LIST[$idx]{location});

  my $scope = $COMPONENT_LIST[$idx]{scope};
  if ($scope eq "SHARED") {
    return;
  }

  my $root = get_distrib_root($COMPONENT_LIST[$idx]{location});

  #
  # remove from the root of each diskless
  # 
  if ($COMPONENT_LIST[$idx]{patch_root} ne "") {
    remove_nmen_patch($node_id, 
		      $COMPONENT_LIST[$idx]{patch_root},
		      "DISKLESS") ;
  }

  if ($COMPONENT_LIST[$idx]{pkg_root} ne "") {
    remove_nmen_pkg($node_id, 
		    $COMPONENT_LIST[$idx]{pkg_root});
  }

  #
  # remove from the Solaris services (usr_specific)
  # during a full cluster install
  #
  if ($INSTALL_TYPE eq "CLUSTER") {

    if ($COMPONENT_LIST[$idx]{patch_usr} ne "") {
      remove_nmen_patch($node_id, 
			$COMPONENT_LIST[$idx]{patch_usr},
			"SOLARIS") ;
    }

    if ($COMPONENT_LIST[$idx]{pkg_usr} ne "") {
      remove_men_pkg($node_id, 
		      $COMPONENT_LIST[$idx]{pkg_usr}, 
		      "USR_SPECIFIC");
    }
  }    
}

#------------------------------------------------------------------------------
#
# OS RELEASE AND PRODUCT MANAGEMENT
#
#------------------------------------------------------------------------------

#------------------------------------------------------------------------------
#
# extract the release info from a file
#
#------------------------------------------------------------------------------

sub get_release {

  my ($release_file) = @_;

  my $release = qx{$GREP "Solaris" $release_file};

  $release =~ s/^[\s\t]*//;
  $release =~ s/[\s\t]*$//;
  chomp($release);

  return $release;
}

#------------------------------------------------------------------------------
#
# is the release a valid release ?
#
#------------------------------------------------------------------------------

sub check_release {

  my ($release) = @_ ;

  for my $i (0..$#OS_LIST) {
    if ($OS_LIST[$i]{idstring} eq $release) {
      return $TRUE;
    }
  }
  
  return $FALSE;
}

#------------------------------------------------------------------------------
#
# get the release index
#
#------------------------------------------------------------------------------

sub get_release_idx {

  my ($release) = @_ ;

  for my $i (0..$#OS_LIST) {
    if ($OS_LIST[$i]{idstring} eq $release) {
      return $i;
    }
  }
}

#------------------------------------------------------------------------------
#
# initialize all the information related to the distribution
#
#------------------------------------------------------------------------------

sub init_distrib {

  my $idx = get_location_idx("SOLARIS");

  $SOLARIS_PACKAGE_SUBDIR = $LOCATION_LIST[$idx]{package};
  $SOLARIS_PATCH_SUBDIR   = $LOCATION_LIST[$idx]{patch};

  $idx = get_location_idx("NHAS");
					       
  $NHAS_PACKAGE_SUBDIR    = $LOCATION_LIST[$idx]{package};
  $NHAS_PATCH_SUBDIR      = $LOCATION_LIST[$idx]{patch};

  # diskless environment definition for service, packages and patches
  if (! $USE_DCSS) {
    $SERVICE_USR_PATH            = "/export/$TARGET_OS/usr_sparc.all";
    $SERVICE_USR_BASEDIR         = "default";
  } else {
    $SERVICE_USR_PATH            = "/export/exec/$TARGET_OS/";
    $SERVICE_USR_BASEDIR         = "default";
  }
}

{
}
